---
author: Tim Izzo
paging: Slide %d / %d
---

# C'est quoi le Web sémantique ?

- Réseau d'informations structurées et typées compréhensibles par les machines
- S’inscrit dans la logique de l’Open-Data mais bien plus ambitieux
- Permet une utilisation standardisée et **interconnectée** des données
- Permet de faire des rapprochements entre plusieurs sources de données / plusieurs sujets
- Considère une distribution de la données / des serveurs plutôt que de la centralisation
- En gros, permet de construire un grand réseau de connaissance étalé sur Internet
- Poussé depuis 20 ans par Tim Berners-Lee [TED](https://www.ted.com/talks/tim_berners_lee_the_next_web)

---

# Différence avec HTML

- HTML est orienté _présentation des données_, à destination des humains
- On peut faire du HTML sémantique pour identifier le contenu de nos pages
- HTML a certaines limites car contextuel à une page (gestion de la navigation, contenus riches et diverses)

```html
<!DOCTYPE html>
<html>
  <head>
    <title>Titre du document</title>
  </head>
  <body>
    <article>
      <h1>Titre de l'article</h1>
      <p>Texte de l'article</p>
    </article>
  </body>
</html>
```

---

# L'existant

Il existe déjà beaucoup de choses et on en utilise plein sans le savoir.

- Plusieurs initiatives pour rendre Wikipedia sémantique: https://www.dbpedia.org/
- Les données OpenGraph sont des données sémantiques consommées par les search engines
- La Confédération Suisse a un service intéressant qui cherche à croiser les données [LINDAS](La Confédération Suisse a un service intéressant qui cherche à croiser les données (LINDAS)

> Aujourd'hui, c'est encore très technique et plutôt complexe, même si ça existe depuis 20 ans.

---

# Le turfu

- La volonté de TBL est de créer le GGG (Giant Global Graph), un nouveau Web qui met l'accent sur les données
- Les GAFAM utilisent à fond les données sémantiques mais pour leur propre gueule
  - Google Search, Google Maps,...
  - Facebook est un pionner en la matière
  - Assistants intelligents (Google Assistant, Siri, Alexa,...)
  - IA
- Des initiatives comme SOLID ou Semapps visent à démocratiser l'utilisation tout en prônant un Web distribué

---

# À quoi ça ressemble ?

## Format de données

Le standard le plus utilisé est RDF, formaté en XML ou JSON.

```xml
<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
   xmlns:foaf="http://xmlns.com/foaf/0.1/"
   xmlns:rss="http://purl.org/rss/1.0/"
   xmlns:dc="http://purl.org/dc/elements/1.1/">
 	<foaf:Person rdf:about="http://example.net/Paul_Dupont">
 		<foaf:name>Paul Dupont</foaf:name>
 		<foaf:img rdf:resource="http://example.net/Paul_Dupont.jpg"/>
 		<foaf:knows rdf:resource="http://example.net/Pierre_Dumoulin"/>
 	</foaf:Person>
 	<foaf:Person rdf:about="http://example.net/Pierre_Dumoulin">
 		<foaf:name>Pierre Dumoulin</foaf:name>
 		<foaf:img rdf:resource="http://example.net/Pierre_Dumoulin.jpg"/>
 	</foaf:Person>
 	<foaf:Image rdf:about="http://example.net/Paul_Dupont.jpg">
 		<dc:description>Photo d'identité de Paul Dupont</dc:description>
 	</foaf:Image>
 	<foaf:Image rdf:about="http://example.net/Pierre_Dumoulin.jpg">
 		<dc:description>Photo d'identité de Pierre Dumoulin</dc:description>
 	</foaf:Image>
 </rdf:RDF>
```

---

#

## Requêtage

Il existe un language de query qui permet de requêter des données. Inspiré de SQL.

```sql
 PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
 PREFIX foaf: <http://xmlns.com/foaf/0.1/>
 PREFIX dc: <http://purl.org/dc/elements/1.1/>
 SELECT DISTINCT ?nom ?image ?description
 WHERE {
 	?personne rdf:type foaf:Person.
 	?personne foaf:name ?nom.
 	?image rdf:type foaf:Image.
 	?personne foaf:img ?image.
 	?image dc:description ?description
 }
```

Cela fonctionne avec des triplets "sujet prédicat objet" et nécessite une base de données de type _triplestore_.

---

# Opportunités pour nous

##### Créer des données sémantiques à partir de R-21 et les proposer à LINDAS

C’est ce qui me semble le plus riche en terme de données sémantiques car regroupe plusieurs corps de métiers et peut intéresser d’autres pour des décisions politiques ou initiatives communales.

##### Créer des données sémantiques à partir de Caroster et s'interconnecter avec d'autres

Si on trouve des données sur la géocalisation d’autres transports, on peut proposer des choses plus complètes.

> Par exemple: comment se déplacer d’un point un A à un point B en utilisant que des moyens de transports de type “Covoiturage”, “Bus”, “Train”.

Google fait ça avec Gmaps et c’est forcément sur une base sémantique afin d'interconnecter toutes les sources.

##### Mettre une couche de sémantique sur nos données internes

Les informations qui nous sont nécessaires sont splitées sur plusieurs Saas et serveurs.
Cela nous demande un effort pour assurer une cohérence de l'ensemble (que l'on fait à travers Notion).

On pourrait revoir la manière dont on traite nos données pour une approche plus sémantique.
Cela nous permettrait d'avoir des métriques plus poussées et plus facilement (data lake).
C'est également une solution pour avoir un système d'archive indexée dans laquelle on peut rapidement trouver ce qu'on veut.

##### Publier les infos d'EVOSPE

EVOSPE a des données hyper intéressantes au niveau communal, cantonal et même fédéral (LINDAS) mais il faut leur accord pour faire quelque chose.
Peut être en leur montrant l'intérêt ?

Autre point, on pourrait "sémantiser" toutes les données et mettre un SPARQL dessus pour avoir un système de remonté d'informations puissants.
Mais il faut voir le besoin avant tout.

##### Decidim

Permettrait de cartographier la communauté Decidim et savoir qui fait quoi où. Comme ce qui est fait sur Kumu mais avec des données utilisables plus largement.
Semapps+Flod.io permettrait d'avoir une vu comme Kumu.

L'ontologie [PAIR](https://www.virtual-assembly.org/ontologie-pair/) est pratique pour ça.

##### Cartographier le réseau d'Octree

Dans la même idée que Decidim, on pourrait cartographier nos projets / clients / partenaires afin de mieux
distinguer des opportunités ou rapprochements et donner de la visbilité.

---

# Comment avancer ?

- Le Web sémantique est un outil et pas un objectif en soi: il faut un but clair avant de faire quelque chose
- C'est un nouveau groupe de compétences qu'on ne maîtrise pas (encore)
- Cela peut nous donner une approche moderne et nouvelle sur la manière de considérer nos données d'applications mais aussi internes, notamment si on veut rendre
  nos apps interopérables.

- Monter en compétence sur RDF, LPD et SPARQL
- Alimenter Archipel (Semapps)
- Faire un POC sur un projet à nous
- Creuser Ontologie PAIR => https://www.virtual-assembly.org/ontologie-pair/
- Monter un pod ou un semapps à nous (avec données d'apps + internes)

---

# Démo Semapps

=> Créer le projet Voca sur [https://archipel.assemblee-virtuelle.org/](https://archipel.assemblee-virtuelle.org/)

Les données sont ensuite utilisables depuis https://data.virtual-assembly.org/ (endpoints LDP).

Ou en version plus graphique: https://archipel.flod.io/

---

# Ressources intéressantes

- Présentation SOLID et Semapps => https://pad.lescommuns.org/p/SlideInteroperabiliteSemapps#/
- Présentation de SOLID => https://pad.lescommuns.org/p/SlideSolid
- Présentation d'un projet national utilisant des Semapps: https://pad.lescommuns.org/p/SlideCarto4CH#/
- Présentation d'Archipel => https://www.virtual-assembly.org/wp-content/uploads/2022/01/Archipelago-1.pdf
- Ontologie PAIR de l'Assemblée Virtuelle : https://www.virtual-assembly.org/ontologie-pair/

---

Voir le contenu de ces slides: `ssh 5ika.ch -p 1917 cat slides/semantic.md -c`
